@@ -1,6 +1,5 @@
-/* $Id: gucharmap-search-dialog.c 1331 2007-04-26 12:56:08Z chpe $ */
 /*
- * Copyright (c) 2004 Noah Levitt
+ * Copyright © 2004 Noah Levitt
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,22 +13,20 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
  */
 
-#include "config.h"
+#include <config.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
 #include "gucharmap-search-dialog.h"
 #include "gucharmap-window.h"
-#include "gucharmap-unicode-info.h"
-#include "gucharmap-marshal.h"
-#include "gucharmap-intl.h"
 
 #define GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), gucharmap_search_dialog_get_type (), GucharmapSearchDialogPrivate))
 
-GdkCursor * _gucharmap_window_progress_cursor ();
+#define I_(string) g_intern_static_string (string)
 
 enum
 {
@@ -38,8 +35,7 @@ enum
   NUM_SIGNALS
 };
 
-static guint gucharmap_search_dialog_signals[NUM_SIGNALS] = { 0, 0 };
-static GObjectClass *parent_class;
+static guint gucharmap_search_dialog_signals[NUM_SIGNALS];
 
 enum
 {
@@ -89,6 +85,10 @@ struct _GucharmapSearchDialogPrivate
   GtkWidget             *next_button;
 };
 
+static void gucharmap_search_dialog_class_init (GucharmapSearchDialogClass *klass);
+static void gucharmap_search_dialog_init       (GucharmapSearchDialog *dialog);
+
+G_DEFINE_TYPE (GucharmapSearchDialog, gucharmap_search_dialog, GTK_TYPE_DIALOG)
 
 static const gchar *
 utf8_strcasestr (const gchar *haystack, 
@@ -380,8 +380,8 @@ static gboolean
 idle_search (GucharmapSearchDialog *search_dialog)
 {
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
-  GTimer *timer = g_timer_new ();
   gunichar wc;
+  GTimer *timer;
 
   /* search without leading and tailing spaces */
   /* with "match whole word" option, there's no need for leading and tailing spaces */
@@ -389,6 +389,8 @@ idle_search (GucharmapSearchDialog *search_dialog)
   if (quick_checks_before (priv->search_state))
     return FALSE;
 
+  timer = g_timer_new ();
+
   do
     {
       priv->search_state->curr_index = (priv->search_state->curr_index + priv->search_state->increment + priv->search_state->list_num_chars) % priv->search_state->list_num_chars;
@@ -402,6 +404,7 @@ idle_search (GucharmapSearchDialog *search_dialog)
       if (priv->search_state->search_string_value != -1 && priv->search_state->curr_index == priv->search_state->search_string_value)
 	{
 	  priv->search_state->found_index = priv->search_state->curr_index;
+          g_timer_destroy (timer);
 	  return FALSE;
 	}
 
@@ -409,14 +412,20 @@ idle_search (GucharmapSearchDialog *search_dialog)
       if (matches (search_dialog, wc, priv->search_state->search_string_nfd, priv->search_state->annotations))
         {
           priv->search_state->found_index = priv->search_state->curr_index;
+          g_timer_destroy (timer);
           return FALSE;
         }
 
       if (g_timer_elapsed (timer, NULL) > 0.050)
-        return TRUE;
+        {
+          g_timer_destroy (timer);
+          return TRUE;
+        }
     }
   while (priv->search_state->curr_index != priv->search_state->start_index);
 
+  g_timer_destroy (timer);
+
   if (quick_checks_after (priv->search_state))
     return FALSE;
 
@@ -446,9 +455,10 @@ gucharmap_search_state_get_found_char (GucharmapSearchState *search_state)
 static void
 gucharmap_search_state_free (GucharmapSearchState *search_state)
 {
+  g_object_unref (search_state->list);
   g_free (search_state->search_string_nfd_temp);
   g_free (search_state->search_string_nfc);
-  g_free (search_state);
+  g_slice_free (GucharmapSearchState, search_state);
 }
 
 /**
@@ -466,7 +476,7 @@ gucharmap_search_state_free (GucharmapSearchState *search_state)
  * Return value: the new #GucharmapSearchState.
  **/
 static GucharmapSearchState * 
-gucharmap_search_state_new (const GucharmapCodepointList *list, 
+gucharmap_search_state_new (GucharmapCodepointList       *list,
                             const gchar                  *search_string, 
                             gint                          start_index, 
                             GucharmapDirection            direction, 
@@ -478,9 +488,9 @@ gucharmap_search_state_new (const GucharmapCodepointList *list,
 
   g_assert (direction == GUCHARMAP_DIRECTION_BACKWARD || direction == GUCHARMAP_DIRECTION_FORWARD);
 
-  search_state = g_new (GucharmapSearchState, 1);
+  search_state = g_slice_new (GucharmapSearchState);
 
-  search_state->list = (GucharmapCodepointList *) list;
+  search_state->list = g_object_ref (list);
   search_state->list_num_chars = gucharmap_codepoint_list_get_last_index (search_state->list) + 1;
 
   search_state->search_string = g_strdup (search_string);
@@ -543,43 +553,21 @@ information_dialog (GucharmapSearchDialog *search_dialog,
                     const gchar           *message)
 {
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
-  GtkWidget *dialog, *hbox, *icon, *label;
+  GtkWidget *dialog;
 
-  /* follow hig guidelines */
-  dialog = gtk_dialog_new ();
+  dialog = gtk_message_dialog_new (GTK_WIDGET_VISIBLE (search_dialog) ?
+                                     GTK_WINDOW (search_dialog) :
+                                     GTK_WINDOW (priv->guw),
+                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_INFO,
+                                   GTK_BUTTONS_OK,
+                                   "%s", message);
   gtk_window_set_title (GTK_WINDOW (dialog), _("Information"));
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
-  gtk_window_set_icon (GTK_WINDOW (dialog), gtk_window_get_icon (GTK_WINDOW (search_dialog)));
+  gtk_window_set_icon_name (GTK_WINDOW (dialog), GUCHARMAP_ICON_NAME);
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-  if (GTK_WIDGET_VISIBLE (search_dialog))
-    gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (search_dialog));
-  else
-    gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (priv->guw));
-
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
-
-  hbox = gtk_hbox_new (FALSE, 12);
-  gtk_widget_show (hbox);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
-
-  icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
-  gtk_widget_show (icon);
-  gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
-
-  label = gtk_label_new (message);
-  gtk_widget_show (label);
-  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
   g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
 
-  gtk_widget_show (dialog);
+  gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static void
@@ -593,11 +581,10 @@ search_completed (GucharmapSearchDialog *search_dialog)
   g_signal_emit (search_dialog, gucharmap_search_dialog_signals[SEARCH_FINISH], 0, found_char);
 
   if (found_char == (gunichar)(-1))
-    information_dialog (search_dialog, _("Not found."));
-  else
     {
-      gtk_widget_set_sensitive (priv->prev_button, TRUE);
-      gtk_widget_set_sensitive (priv->next_button, TRUE);
+      information_dialog (search_dialog, _("Not found."));
+      gtk_widget_set_sensitive (priv->prev_button, FALSE);
+      gtk_widget_set_sensitive (priv->next_button, FALSE);
     }
 
   gdk_window_set_cursor (GTK_WIDGET (search_dialog)->window, NULL);
@@ -625,12 +612,16 @@ _gucharmap_search_dialog_fire_search (GucharmapSearchDialog *search_dialog,
   gunichar start_char;
   gint start_index;
 
+  if (priv->search_state && priv->search_state->searching) /* Already searching */
+    return;
+
   GdkCursor *cursor = _gucharmap_window_progress_cursor ();
   gdk_window_set_cursor (GTK_WIDGET (search_dialog)->window, cursor);
   gdk_cursor_unref (cursor);
 
-
-  list = (GucharmapCodepointList *) gucharmap_chapters_get_book_codepoint_list (gucharmap_charmap_get_chapters (priv->guw->charmap));
+  list = gucharmap_charmap_get_book_codepoint_list (priv->guw->charmap);
+  if (!list)
+    return;
 
   if (priv->search_state == NULL
       || list != priv->search_state->list
@@ -641,7 +632,7 @@ _gucharmap_search_dialog_fire_search (GucharmapSearchDialog *search_dialog,
       if (priv->search_state)
         gucharmap_search_state_free (priv->search_state);
 
-      start_char = gucharmap_table_get_active_character (priv->guw->charmap->chartable);
+      start_char = gucharmap_charmap_get_active_character (priv->guw->charmap);
       start_index = gucharmap_codepoint_list_get_index (list, start_char);
       priv->search_state = gucharmap_search_state_new (list, gtk_entry_get_text (GTK_ENTRY (priv->entry)),
 			      start_index, direction,
@@ -650,20 +641,19 @@ _gucharmap_search_dialog_fire_search (GucharmapSearchDialog *search_dialog,
     }
   else
     {
-      start_char = gucharmap_table_get_active_character (priv->guw->charmap->chartable);
+      start_char = gucharmap_charmap_get_active_character (priv->guw->charmap);
       priv->search_state->start_index = gucharmap_codepoint_list_get_index (list, start_char);
       priv->search_state->curr_index = priv->search_state->start_index;
       priv->search_state->increment = direction;
     }
 
-  gtk_widget_set_sensitive (priv->prev_button, FALSE);
-  gtk_widget_set_sensitive (priv->next_button, FALSE);
-
   priv->search_state->searching = TRUE;
   priv->search_state->strings_checked = 0;
 
   g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, (GSourceFunc) idle_search, search_dialog, (GDestroyNotify) search_completed);
   g_signal_emit (search_dialog, gucharmap_search_dialog_signals[SEARCH_START], 0);
+
+  g_object_unref (list);
 }
 
 void
@@ -708,17 +698,12 @@ entry_changed (GtkObject             *object,
                GucharmapSearchDialog *search_dialog)
 {
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
+  gboolean is_empty;
 
-  if (_entry_is_empty (GTK_ENTRY (priv->entry)))
-    {
-      gtk_widget_set_sensitive (priv->prev_button, FALSE);
-      gtk_widget_set_sensitive (priv->next_button, FALSE);
-    }
-  else
-    {
-      gtk_widget_set_sensitive (priv->prev_button, TRUE);
-      gtk_widget_set_sensitive (priv->next_button, TRUE);
-    }
+  is_empty = _entry_is_empty (GTK_ENTRY (priv->entry));
+      
+  gtk_widget_set_sensitive (priv->prev_button, !is_empty);
+  gtk_widget_set_sensitive (priv->next_button, !is_empty);
 }
 
 static void 
@@ -823,7 +808,7 @@ gucharmap_search_dialog_finalize (GObject *object)
   if (priv->search_state)
     gucharmap_search_state_free (priv->search_state);
 
-  parent_class->finalize (object);
+  G_OBJECT_CLASS (gucharmap_search_dialog_parent_class)->finalize (object);
 }
 
 static void
@@ -831,53 +816,22 @@ gucharmap_search_dialog_class_init (GucharmapSearchDialogClass *clazz)
 {
   g_type_class_add_private (clazz, sizeof (GucharmapSearchDialogPrivate));
 
-  parent_class = (GObjectClass *) g_type_class_peek_parent (clazz);
-
   G_OBJECT_CLASS (clazz)->finalize = gucharmap_search_dialog_finalize;
 
   clazz->search_start = NULL;
   clazz->search_finish = NULL;
 
   gucharmap_search_dialog_signals[SEARCH_START] =
-      g_signal_new ("search-start", gucharmap_search_dialog_get_type (), G_SIGNAL_RUN_FIRST, 
+      g_signal_new (I_("search-start"), gucharmap_search_dialog_get_type (), G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET (GucharmapSearchDialogClass, search_start), NULL, NULL, 
-                    _gucharmap_marshal_VOID__VOID, G_TYPE_NONE, 0);
+                    g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   gucharmap_search_dialog_signals[SEARCH_FINISH] =
-      g_signal_new ("search-finish", gucharmap_search_dialog_get_type (), G_SIGNAL_RUN_FIRST, 
+      g_signal_new (I_("search-finish"), gucharmap_search_dialog_get_type (), G_SIGNAL_RUN_FIRST, 
                     G_STRUCT_OFFSET (GucharmapSearchDialogClass, search_finish), NULL, NULL, 
-                    _gucharmap_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
-
-  _gucharmap_intl_ensure_initialized ();
+                    g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
 }
 
-GType
-gucharmap_search_dialog_get_type (void)
-{
-  static GType t = 0;
-
-  if (t == 0)
-    {
-      static const GTypeInfo type_info =
-        {
-          sizeof (GucharmapSearchDialogClass),
-          NULL,
-          NULL,
-          (GClassInitFunc) gucharmap_search_dialog_class_init,
-          NULL,
-          NULL,
-          sizeof (GucharmapSearchDialog),
-          0,
-          (GInstanceInitFunc) gucharmap_search_dialog_init,
-          NULL
-        };
-
-      t = g_type_register_static (gtk_dialog_get_type (), "GucharmapSearchDialog", &type_info, 0);
-    }
-
-  return t;
-}
-
-GtkWidget * 
+GtkWidget *
 gucharmap_search_dialog_new (GucharmapWindow *guw)
 {
   GucharmapSearchDialog *search_dialog = g_object_new (gucharmap_search_dialog_get_type (), NULL);
