@@ -1,6 +1,6 @@
-/* $Id: gucharmap.h 1493 2007-12-12 19:33:26Z chpe $ */
 /*
- * Copyright (c) 2004 Noah Levitt
+ * Copyright © 2004 Noah Levitt
+ * Copyright © 2008 Christian Persch
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -20,16 +20,32 @@
 #ifndef GUCHARMAP_H
 #define GUCHARMAP_H
 
-#include <gucharmap/gucharmap-table.h>
-#include <gucharmap/gucharmap-charmap.h>
-#include <gucharmap/gucharmap-window.h>
-#include <gucharmap/gucharmap-unicode-info.h>
-#include <gucharmap/gucharmap-mini-fontsel.h>
+#define __GUCHARMAP_GUCHARMAP_H_INSIDE__
+
+#define GUCHARMAP_VERSION_MAJOR (2)
+#define GUCHARMAP_VERSION_MINOR (23)
+#define GUCHARMAP_VERSION_MICRO (4)
+
+#define GUCHARMAP_CHECK_VERSION(major,minor,micro) \
+  (GUCHARMAP_VERSION_MAJOR > (major) || \
+   (GUCHARMAP_VERSION_MAJOR == (major) && GUCHARMAP_VERSION_MINOR > (minor)) || \
+   (GUCHARMAP_VERSION_MAJOR == (major) && GUCHARMAP_VERSION_MINOR == (minor) && GUCHARMAP_VERSION_MICRO >= (micro)))
+
 #include <gucharmap/gucharmap-codepoint-list.h>
+#include <gucharmap/gucharmap-block-codepoint-list.h>
 #include <gucharmap/gucharmap-script-codepoint-list.h>
-#include <gucharmap/gucharmap-chapters.h>
-#include <gucharmap/gucharmap-block-chapters.h>
-#include <gucharmap/gucharmap-script-chapters.h>
-#include <gucharmap/gucharmap-search-dialog.h>
+
+#include <gucharmap/gucharmap-chapters-model.h>
+#include <gucharmap/gucharmap-block-chapters-model.h>
+#include <gucharmap/gucharmap-script-chapters-model.h>
+
+#include <gucharmap/gucharmap-chartable.h>
+#include <gucharmap/gucharmap-charmap.h>
+
+#include <gucharmap/gucharmap-type-builtins.h>
+
+#include <gucharmap/gucharmap-unicode-info.h>
+
+#undef __GUCHARMAP_GUCHARMAP_H_INSIDE__
 
 #endif /* #ifndef GUCHARMAP_H */
